"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.toNumber = exports.scrapeBetaChangesets = exports.scrapeArchivedChangesets = exports.getBetaChangeset = exports.getArchivedChangeset = exports.getUnityChangeset = void 0;
const node_fetch_1 = __importDefault(require("node-fetch"));
const jsdom_1 = require("jsdom");
const unityChangeset_1 = require("./unityChangeset");
const UNITY_ARCHIVE_URL = 'https://unity3d.com/get-unity/download/archive';
const getDocumentFromUrl = async (archiveUrl) => {
    const response = await node_fetch_1.default(archiveUrl);
    const html = await response.text();
    return new jsdom_1.JSDOM(html).window.document;
};
const getUnityChangeset = async (version) => {
    const match = version.match(/^\d{4}\.\d+\.\d+(a|b|f)\d+$/);
    switch (match === null || match === void 0 ? void 0 : match[1]) {
        case 'f':
            return await exports.getArchivedChangeset(version);
        case 'a':
            return await exports.getBetaChangeset(version, 'alpha');
        case 'b':
            return await exports.getBetaChangeset(version, 'beta');
        default:
            throw Error(`The given life-cycle '${match === null || match === void 0 ? void 0 : match[1]}' (in ${version}) was not supported`);
    }
};
exports.getUnityChangeset = getUnityChangeset;
const getArchivedChangeset = async (version) => {
    const versions = await exports.scrapeArchivedChangesets();
    return versions.filter(c => c.version === version)[0];
};
exports.getArchivedChangeset = getArchivedChangeset;
const getBetaChangeset = async (version, channel) => {
    const document = await getDocumentFromUrl(`https://unity3d.com/unity/${channel}/${version}`);
    return Array.from(document.querySelectorAll('a[href]'))
        .map(a => a.getAttribute('href'))
        .filter(href => unityChangeset_1.UnityChangeset.isValid(href))
        .map(href => unityChangeset_1.UnityChangeset.createFromHref(href))[0];
};
exports.getBetaChangeset = getBetaChangeset;
const scrapeArchivedChangesets = async () => {
    const document = await getDocumentFromUrl(UNITY_ARCHIVE_URL);
    return Array.from(document.querySelectorAll('a[href]'))
        .map(a => a.getAttribute('href'))
        .filter(href => unityChangeset_1.UnityChangeset.isValid(href))
        .map(href => unityChangeset_1.UnityChangeset.createFromHref(href));
};
exports.scrapeArchivedChangesets = scrapeArchivedChangesets;
const scrapeBetaChangesets = async () => {
    var _a, _b;
    const response = await node_fetch_1.default("https://unity3d.com/unity/beta/latest.xml");
    const raw = await (response === null || response === void 0 ? void 0 : response.text());
    const results = new Set();
    (_b = (_a = raw === null || raw === void 0 ? void 0 : raw.match(/unityhub:\/\/([^\/]*)\/([0-9a-f]*)/g)) === null || _a === void 0 ? void 0 : _a.map(m => m.match(/unityhub:\/\/([^\/]*)\/([0-9a-f]*)/))) === null || _b === void 0 ? void 0 : _b.forEach(m => {
        if (m == null)
            return;
        results.add(new unityChangeset_1.UnityChangeset(m[1], m[2]));
    });
    return Array.from(results);
};
exports.scrapeBetaChangesets = scrapeBetaChangesets;
const toNumber = (version, max) => {
    return unityChangeset_1.UnityChangeset.toNumber(version, max);
};
exports.toNumber = toNumber;
//# sourceMappingURL=index.js.map